#ifndef _CHRONOPTICS_TOF_USER_CONFIG_H_
#define _CHRONOPTICS_TOF_USER_CONFIG_H_

#include <chronoptics/tof/camera_config.h>
#include <chronoptics/tof/camera.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The integration time as fixed presets
 */
enum tof_integration_time {
  TOF_INTEGRATION_TIME_SHORT = 0, /** 250 us */
  TOF_INTEGRATION_TIME_MEDIUM = 1, /** 500 us */
  TOF_INTEGRATION_TIME_LONG = 2, /** 1000us */
};

/** The different imaging environments
 */
enum tof_imaging_environment {
  TOF_IMAGING_ENVIRONMENT_INSIDE = 0, /** Inside buildings */
  TOF_IMAGING_ENVIRONMENT_SUNLIGHT = 1, /** When bright sunlight is present */
};

/** The optimization of the camera configuration
 */
enum tof_strategy {
  TOF_STRATEGY_BALANCED = 0, /** The balanced configuration */
  TOF_STRATEGY_SPEED = 1, /** Optimize for moving objects */
  TOF_STRATEGY_ACCURACY = 2, /** Optimize for high accuracy */
};

/** Simple interface for camera configuration generation
 */
typedef struct tof_user_config* tof_user_config_t;

/** Destruct tof_user_config */
TOF_EXPORT void tof_user_config_delete(tof_user_config_t ptr);

/** Create default user config
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_user_config_t tof_user_config_new_default(tof_error_t *error);

/** Convert to a camera configuration
 * @param ptr Pointer to class
 * @param camera The camera
 * @param error Pointer to error
 * @return The camera configuration
 */
TOF_EXPORT tof_camera_config_t tof_user_config_to_camera_config(const tof_user_config_t ptr, tof_camera_t camera, tof_error_t *error);

/** Get the camera operating environment
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Imaging environment
 */
TOF_EXPORT enum tof_imaging_environment tof_user_config_get_environment(const tof_user_config_t ptr, tof_error_t *error);

/** Set the camera operating environment
 * @param ptr Pointer to class
 * @param imaging_environment Imaging environment
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_environment(tof_user_config_t ptr, enum tof_imaging_environment imaging_environment, tof_error_t *error);

/** Get the maximum distance photons reflect from
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Maximum distance
 */
TOF_EXPORT float tof_user_config_get_max_distance(const tof_user_config_t ptr, tof_error_t *error);

/** Set the maximum distance photons reflect from
 * @param ptr Pointer to class
 * @param max_distance Maximum distance
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_max_distance(tof_user_config_t ptr, float max_distance, tof_error_t *error);

/** Get the depth frame rate
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The depth fps
 */
TOF_EXPORT float tof_user_config_get_fps(const tof_user_config_t ptr, tof_error_t *error);

/** Set the depth frame rate
 * @param ptr Pointer to class
 * @param fps The depth fps
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_fps(tof_user_config_t ptr, float fps, tof_error_t *error);

/** Get if lens flare is expected in the scene
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return if lens flare is present
 */
TOF_EXPORT bool tof_user_config_get_lens_flare(const tof_user_config_t ptr, tof_error_t *error);

/** Set if lens flare is expected in the scene
 * @param ptr Pointer to class
 * @param lens_flare if lens flare is present
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_lens_flare(tof_user_config_t ptr, bool lens_flare, tof_error_t *error);

/** Get if there are translucent objects in the scene
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return If translucent objects
 */
TOF_EXPORT bool tof_user_config_get_translucent(const tof_user_config_t ptr, tof_error_t *error);

/** Set if there are translucent objects in the scene
 * @param ptr Pointer to class
 * @param translucent If translucent objects
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_translucent(tof_user_config_t ptr, bool translucent, tof_error_t *error);

/** Get high dynamic range enabled. Enable this when pixel saturation occurs.
 * This will only happen for retroreflectors or reflective objects close (<20cm)
 * to the camera
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enable high dynamic range
 */
TOF_EXPORT bool tof_user_config_get_hdr(const tof_user_config_t ptr, tof_error_t *error);

/** Set high dynamic range enabled. Enable this when pixel saturation occurs.
 * This will only happen for retroreflectors or reflective objects close (<20cm)
 * to the camera
 * @param ptr Pointer to class
 * @param hdr Enable high dynamic range
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_hdr(tof_user_config_t ptr, bool hdr, tof_error_t *error);

/** Get the integration time
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The integration time
 */
TOF_EXPORT enum tof_integration_time tof_user_config_get_integration_time(const tof_user_config_t ptr, tof_error_t *error);

/** Set the integration time
 * @param ptr Pointer to class
 * @param int_time The integration time
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_integration_time(tof_user_config_t ptr, enum tof_integration_time int_time, tof_error_t *error);

/** Get what the configuration is being optimized for
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The user configuration strategy
 */
TOF_EXPORT enum tof_strategy tof_user_config_get_strategy(const tof_user_config_t ptr, tof_error_t *error);

/** Set what the configuration is being optimized for
 * @param ptr Pointer to class
 * @param strategy The user configuration strategy
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_strategy(tof_user_config_t ptr, enum tof_strategy strategy, tof_error_t *error);

/** Get The camera channel, increment to support multi-camera usage
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The channel number, set to 0, otherwise increment with number of
 * cameras.
 */
TOF_EXPORT int32_t tof_user_config_get_channel(const tof_user_config_t ptr, tof_error_t *error);

/** Set The camera channel, increment to support multi-camera usage
 * @param ptr Pointer to class
 * @param channel The channel number, set to 0, otherwise increment with number
 * of cameras.
 * @param error Pointer to error
 */
TOF_EXPORT void tof_user_config_set_channel(tof_user_config_t ptr, int32_t channel, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
